"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendPrivyTransaction = sendPrivyTransaction;
const viem_1 = require("viem");
const actions_1 = require("viem/actions");
const chains_1 = require("viem/chains");
const utils_1 = require("viem/utils");
const eip712_js_1 = require("../eip712.js");
const account_js_1 = require("../errors/account.js");
const ALLOWED_CHAINS = [chains_1.abstractTestnet.id];
function convertCallsToPermissionlessCalls(calls) {
    return (calls || []).map((call) => ({
        to: call.target,
        value: call.value.toString(),
        data: call.callData,
    }));
}
function convertBigIntToString(obj) {
    if (typeof obj === 'bigint') {
        return obj.toString();
    }
    if (typeof obj !== 'object' || obj === null) {
        return obj;
    }
    if (Array.isArray(obj)) {
        return obj.map(convertBigIntToString);
    }
    return Object.fromEntries(Object.entries(obj).map(([key, value]) => [
        key,
        convertBigIntToString(value),
    ]));
}
async function sendPrivyTransaction(client, signerClient, args, useSignerAddress = false) {
    const { account: account_ = client.account, chain = client.chain, ...transaction } = args;
    transaction.type = 'eip712';
    transaction.value = transaction.value
        ? BigInt(transaction.value)
        : 0n;
    if (!account_)
        throw new account_js_1.AccountNotFoundError({
            docsPath: '/docs/actions/wallet/signTransaction',
        });
    const smartAccount = (0, utils_1.parseAccount)(account_);
    const fromAccount = useSignerAddress ? signerClient.account : smartAccount;
    (0, eip712_js_1.assertEip712Request)({
        account: smartAccount,
        chain,
        ...transaction,
    });
    if (!chain || !ALLOWED_CHAINS.includes(chain.id)) {
        throw new viem_1.BaseError('Invalid chain specified');
    }
    if (!chain?.custom?.getEip712Domain)
        throw new viem_1.BaseError('`getEip712Domain` not found on chain.');
    if (!chain?.serializers?.transaction)
        throw new viem_1.BaseError('transaction serializer not found on chain.');
    const chainId = await (0, utils_1.getAction)(client, actions_1.getChainId, 'getChainId')({});
    if (chain !== null)
        (0, utils_1.assertCurrentChain)({
            currentChainId: chainId,
            chain: chain,
        });
    const calls = [
        {
            target: transaction.to,
            allowFailure: false,
            value: BigInt(transaction.value ?? 0),
            callData: transaction.data ?? '0x',
        },
    ];
    const serializedTransaction = convertBigIntToString(transaction);
    const result = (await client.request({
        method: 'privy_sendSmartWalletTx',
        params: [
            fromAccount,
            serializedTransaction,
            convertCallsToPermissionlessCalls(calls),
        ],
    }, { retryCount: 0 }));
    return result;
}
//# sourceMappingURL=sendPrivyTransaction.js.map